// Copyright 2026 The Bazel Authors. All rights reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package com.google.devtools.build.lib.skyframe;

import com.google.devtools.build.lib.skyframe.serialization.autocodec.AutoCodec;
import com.google.devtools.build.skyframe.SkyValue;
import javax.annotation.Nullable;

/**
 * The value of an environmental variable from an environment (client env, action env or repository
 * env). These are invalidated and injected by {@link SequencedSkyframeExecutor}.
 *
 * @param value the value in the client environment or null if unset in the environment.
 */
@AutoCodec
public record EnvironmentVariableValue(@Nullable String value) implements SkyValue {}
